package com.ruoyi.sim.service.impl;

import java.util.List;

import com.ruoyi.common.utils.DateUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.sim.mapper.RealExamFaultMapper;
import com.ruoyi.sim.domain.RealExamFault;

/**
 * 考试故障关联Service业务层处理
 *
 * @author tom
 * @date 2024-12-15
 */
@Service
public class RealExamFaultService {
    @Autowired
    private RealExamFaultMapper realExamFaultMapper;

    /**
     * 查询考试故障关联
     *
     * @param refId 考试故障关联主键
     * @return 考试故障关联
     */
    public RealExamFault selectRealExamFaultByRefId(Long refId) {
        return realExamFaultMapper.selectRealExamFaultByRefId(refId);
    }

    /**
     * 查询考试故障关联列表
     *
     * @param realExamFault 考试故障关联
     * @return 考试故障关联
     */
    public List<RealExamFault> selectRealExamFaultList(RealExamFault realExamFault) {
        return realExamFaultMapper.selectRealExamFaultList(realExamFault);
    }

    /**
     * 新增考试故障关联
     *
     * @param realExamFault 考试故障关联
     * @return 结果
     */
    public int insertRealExamFault(RealExamFault realExamFault) {
        realExamFault.setCreateTime(DateUtils.getNowDate());
        return realExamFaultMapper.insertRealExamFault(realExamFault);
    }

    /**
     * 修改考试故障关联
     *
     * @param realExamFault 考试故障关联
     * @return 结果
     */
    public int updateRealExamFault(RealExamFault realExamFault) {
        realExamFault.setUpdateTime(DateUtils.getNowDate());
        return realExamFaultMapper.updateRealExamFault(realExamFault);
    }

    /**
     * 批量删除考试故障关联
     *
     * @param refIds 需要删除的考试故障关联主键
     * @return 结果
     */
    public int deleteRealExamFaultByRefIds(Long[] refIds) {
        return realExamFaultMapper.deleteRealExamFaultByRefIds(refIds);
    }

    /**
     * 删除考试故障关联信息
     *
     * @param refId 考试故障关联主键
     * @return 结果
     */
    public int deleteRealExamFaultByRefId(Long refId) {
        return realExamFaultMapper.deleteRealExamFaultByRefId(refId);
    }

    // -------------------------------- tom add  --------------------------------

    private static final Logger l = LoggerFactory.getLogger(RealExamFaultService.class);

    /**
     * 初始化状态的 关联列表。
     *
     * @param examId
     * @return
     */
    public List<RealExamFault> listAllType2InitStateByExamId(Long examId) {
        RealExamFault q = new RealExamFault();
        q.setExamId(examId);
        q.setRefType(RealExamFault.REF_TYPE_2);
        q.setRefState(RealExamFault.REF_STATE_INIT);
        return realExamFaultMapper.selectRealExamFaultList(q);
    }

    public List<RealExamFault> listAllType2YesLoopReadState(Long examId) {
        RealExamFault q = new RealExamFault();
        q.setExamId(examId);
        q.setRefType(RealExamFault.REF_TYPE_2);
        q.setRefState(RealExamFault.REF_STATE_LOOP_READ);
        q.setFlag(RealExamFault.FLAG_YES);
        return realExamFaultMapper.selectRealExamFaultList(q);
    }

    public List<RealExamFault> listAllType2Yes(Long examId) {
        RealExamFault q = new RealExamFault();
        q.setExamId(examId);
        q.setRefType(RealExamFault.REF_TYPE_2);
        q.setFlag(RealExamFault.FLAG_YES);
        return realExamFaultMapper.selectRealExamFaultList(q);
    }

    public boolean isAllType2YesStateEqualWritten(long examId) {
        List<RealExamFault> list = listAllType2Yes(examId);
        for (RealExamFault ref : list) {
            if (ref.getRefState() != RealExamFault.REF_STATE_WRITTEN) {
                return false;
            }
        }
        return true;
    }

    public boolean isState(Long refId, String state) {
        RealExamFault f = selectRealExamFaultByRefId(refId);
        if (f == null) {
            return false;
        }
        return state.equals(f.getRefState());
    }

    /**
     * 计算减分。
     *
     * @param refId
     */
    public void calculateMinus(Long refId) {
        RealExamFault f = selectRealExamFaultByRefId(refId);
        if (f == null) {
            throw new RuntimeException("calculateMinus");
        }
        if (!RealExamFault.FLAG_YES.equals(f.getFlag())) {
            throw new RuntimeException("calculateMinus");
        }
        int minus = 0;
        if (RealExamFault.REF_TYPE_1.equals(f.getRefType())) {
            // todo:选择题的算减分
        } else if (RealExamFault.REF_TYPE_2.equals(f.getRefType())) {
            // 模拟器故障的算减分
            if (!f.getChoiceQuestionValue().equals(f.getChoiceAnswerValue())) {
                // 扣0分
                minus = 0;
            } else {
                // 扣25分 故障未排除
                minus = 25;
            }
        }
        f.setMinus(minus);
        // 更新减分数据。
        updateRealExamFault(f);
    }
}
