package com.ruoyi.sim.service.impl;

import java.util.List;
import java.util.Objects;

import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.sim.domain.*;
import com.ruoyi.sim.domain.vo.StudentRealExamPostVo;
import com.ruoyi.sim.domain.vo.StudentRealExamPreVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.sim.mapper.RealExamMapper;
import org.springframework.transaction.annotation.Transactional;

/**
 * 考试Service业务层处理
 *
 * @author tom
 * @date 2024-12-15
 */
@Service
public class RealExamService {
    @Autowired
    private RealExamMapper realExamMapper;

    /**
     * 查询考试
     *
     * @param examId 考试主键
     * @return 考试
     */
    public RealExam selectRealExamByExamId(Long examId) {
        return realExamMapper.selectRealExamByExamId(examId);
    }

    /**
     * 查询考试列表
     *
     * @param realExam 考试
     * @return 考试
     */
    public List<RealExam> selectRealExamList(RealExam realExam) {
        return realExamMapper.selectRealExamList(realExam);
    }

    /**
     * 新增考试
     *
     * @param realExam 考试
     * @return 结果
     */
    public int insertRealExam(RealExam realExam) {
        realExam.setCreateTime(DateUtils.getNowDate());
        return realExamMapper.insertRealExam(realExam);
    }

    /**
     * 修改考试
     *
     * @param realExam 考试
     * @return 结果
     */
    public int updateRealExam(RealExam realExam) {
        realExam.setUpdateTime(DateUtils.getNowDate());
        return realExamMapper.updateRealExam(realExam);
    }

    /**
     * 批量删除考试
     *
     * @param examIds 需要删除的考试主键
     * @return 结果
     */
    public int deleteRealExamByExamIds(Long[] examIds) {
        return realExamMapper.deleteRealExamByExamIds(examIds);
    }

    /**
     * 删除考试信息
     *
     * @param examId 考试主键
     * @return 结果
     */
    public int deleteRealExamByExamId(Long examId) {
        return realExamMapper.deleteRealExamByExamId(examId);
    }

    // -------------------------------- tom add  --------------------------------
    @Autowired
    private StudentService studentService;
    @Autowired
    private SimService simService;
    @Autowired
    private SeatService seatService;
    @Autowired
    private RealExamCollectionService realExamCollectionService;
    @Autowired
    private RealExamFaultService realExamFaultService;
    @Autowired
    private CommSendService commSendService;

    public List<RealExam> listAllStatus(String examStatus) {
        RealExam q = new RealExam();
        q.setExamStatus(examStatus);
        return selectRealExamList(q);
    }

    @Transactional
    public int updateOneState(RealExam re, final String state) {
        RealExam q = selectRealExamByExamId(re.getExamId());
        if (state.equals(RealExam.STATE_SUBMITTED)) {
            // 关联故障list同步锁死。
            realExamFaultService.listAllType2YesLoopReadState(q.getExamId())
                    .forEach(ref -> {
                        ref.setRefState(RealExamFault.REF_STATE_FINISH);
                        realExamFaultService.updateRealExamFault(ref);
                    });
        }
        q.setExamStatus(state);
        return updateRealExam(q);
    }

    /**
     * 计算减分(不包括超时)。汇总到deduction_total_score字段。
     *
     * @param realExamId
     */
    public void calculate(Long realExamId) {
        int minus = 0;
        RealExam re = realExamMapper.selectRealExamByExamId(realExamId);

        // 排除故障部分


        // 维修报告选择部分

        // 超时部分
        int timeoutMinute = 0;
    }

    /**
     * [学生]进入考试。
     *
     * @return
     */
    public AjaxResult studentEnterRealExam() {
        RealExam re = null;
        return AjaxResult.success(re);
    }

    /**
     * [轮询][学生]准备考试界面。
     *
     * @param realExamId
     * @return
     */
    public StudentRealExamPreVo studentPrepareRealExam(Long realExamId) {
        // check
        if (realExamId == null || realExamId == 0) {
            // todo:
        }
        //
        RealExam re = selectRealExamByExamId(realExamId);
        if (re == null) {
            // todo:
        }
        // todo: 日期，不可进入考试。
        //
        // todo: 验证学生登录身份
        Objects.requireNonNull(re);
        RealExamCollection collection =
                realExamCollectionService.selectRealExamCollectionByExamCollectionId(re.getExamCollectionId());
        // check collection
        Sim sim = simService.selectSimBySimId(re.getSimId());
        // check sim
        Student student = studentService.selectStudentByUserId(re.getUserId());
        // check student
        Seat seat = seatService.selectSeatBySeatId(re.getSeatId());
        // check seat
        StudentRealExamPreVo v = new StudentRealExamPreVo();
        v.setRealExam(re);
        v.setRealExamCollection(collection);
        v.setSim(sim);
        v.setStudent(student);
        v.setSeat(seat);
        boolean next = studentPrepareRealExamCheck(v);
        v.setNext(next);
        if (!next) {
            // 执行模拟器通信，让模拟器准备好。
            // async execute.
            commSendService.clearListFaultByRealExamAsync(re);
        }
        return v;
    }

    public boolean studentPrepareRealExamCheck(StudentRealExamPreVo v) {
        if (v == null ||
                v.getRealExam() == null ||
                v.getRealExamCollection() == null ||
                v.getSim() == null ||
                v.getStudent() == null ||
                v.getSeat() == null) {
            return false;
        }
        // todo:在考试日期内。
        // check 一个模拟器的所有选中故障点位都下发成功，准备是否可以
        //
        // todo:??
        // 学生答题中可以再次进入。
        if (v.getRealExam().getExamStatus() == RealExam.STATE_SIM_PREPARE_OK ||
                v.getRealExam().getExamStatus() == RealExam.STATE_ANSWERING ||
                v.getSim().getSimState() == Sim.STATE_ONLINE
        ) {
            return true;
        }
        return false;
    }

    /**
     * [轮询][学生]正在考试界面。
     *
     * @param realExamId
     * @return
     */
    public StudentRealExamPreVo studentIngRealExam(Long realExamId) {
        return null;
    }

    /**
     * [学生]交卷
     *
     * @param realExamId
     * @return
     */
    public AjaxResult studentSubmitRealExam(Long realExamId) {
        // 最后检查一下模拟器状态。
        
        // 最后读取一下模拟器电阻值。
        return AjaxResult.success();
    }

    /**
     * [轮询][学生]结束考试界面。
     *
     * @param realExamId
     * @return
     */
    public AjaxResult studentPostRealExam(Long realExamId) {
        StudentRealExamPostVo vo = new StudentRealExamPostVo();
        return AjaxResult.success(vo);
    }
}
